from qt import *
from utils import *
##~ import libxsltmod
from kdeemul import *
import shutil
import paths

class LHTMLExport(QDialog):
	def __init__(self, parent, lib, albumName):
		QDialog.__init__(self, parent, "Lphoto", True)
		self.lib = lib
		self.albumName = albumName
		self.setCaption(i18n("Create Web Pages"))
		
		gl = QGridLayout(self,4,2,10)
		l = QLabel(i18n("Maximum Size:"), self)
		gl.addWidget(l,0,0)
		self.sizeCombo = QComboBox(self)
		gl.addWidget(self.sizeCombo,0,1)
		
		l = QLabel(i18n("Export Path"), self)
		gl.addWidget(l,1,0)
		hl = QHBoxLayout()
		gl.addLayout(hl,1,1)
		self.exportEdit = QLineEdit(self)
		self.exportEdit.setText(str(os.path.join(lib.libraryPath(),"web pages", albumName)))
		hl.addWidget(self.exportEdit)
		self.exportPathButton = QPushButton("...", self)
		hl.addWidget(self.exportPathButton)
		
		l = QLabel(i18n("Template File"), self)
		gl.addWidget(l,2,0)
		hl = QHBoxLayout()
		gl.addLayout(hl,2,1)
		self.templateEdit = QLineEdit(self)
		self.templateEdit.setText(paths.template("thumb.xsl"))
		hl.addWidget(self.templateEdit)
		self.templateButton = QPushButton("...", self)
		hl.addWidget(self.templateButton)
		
		hl = QHBoxLayout()
		gl.addLayout(hl,3,1)
		hl.addStretch()
		self.exportButton = QPushButton(i18n("Create"), self)
		hl.addWidget(self.exportButton)
		self.exportButton.setDefault(True)
		hl.addSpacing(5)
		self.cancelButton = QPushButton(i18n("Cancel"), self)
		hl.addWidget(self.cancelButton)
		
		# init sizeCombo
		self.sizes = []
		self.sizes.append([i18n("Small (320x240)"), QSize(320,240), 65000])
		self.sizes.append([i18n("Medium (640x480)"), QSize(640,480), 124000])
		self.sizes.append([i18n("Large (1200x900)"), QSize(1200,900), 180000])
		self.sizes.append([i18n("Full (Actual Size)"), None, None])
		
		for size in self.sizes:
			self.sizeCombo.insertItem(size[0])
		# select last item
		self.sizeCombo.setCurrentItem(self.sizeCombo.count()-1)
		
		QObject.connect(self.exportButton, SIGNAL("clicked()"), self, SLOT("accept()"))
		QObject.connect(self.cancelButton, SIGNAL("clicked()"), self, SLOT("reject()"))
		QObject.connect(self.exportPathButton, SIGNAL("clicked()"), self.slotShowPathDialog)
		
	def slotShowPathDialog(self):
		path = KFileDialog.getExistingDirectory(self.exportEdit.text(), None, "", i18n("Select your export directory"))
		if path:
			self.exportEdit.setText(path)

		
	def done(self, result):
		self.size = self.sizes[self.sizeCombo.currentItem()][1]
		self.template = str(self.templateEdit.text())
		self.exportDir = str(self.exportEdit.text())
		return QDialog.done(self, result)
		
	def __xslXform(self, xmlString, templatePath, outputPath):
##~ 		libxsltmod.translate_to_file('f', templatePath, 's', xmlString, outputPath, None)
		tmpEnv = os.environ.get('TMP','/tmp')
		tmpPath = os.path.join(tmpEnv, "lphototmp.xml")
		ouf = open(tmpPath, mode="w")
		ouf.write(xmlString)
		ouf.close()
		os.system("xsltproc -o \"%s\" \"%s\" \"%s\"" % (outputPath, templatePath, tmpPath))

		
	def exportHTML(self, keys):
		# create directory
		thumbDir = os.path.join(self.exportDir, "thumbs")
		LUtils.ensureDirectory(thumbDir)
		imageDir = os.path.join(self.exportDir, "images")
		LUtils.ensureDirectory(imageDir)
		
		indexHTML = os.path.join(self.exportDir, "index.html")
		
		progress = QProgressDialog(i18n("Creating Web Pages"),None,len(keys),None,"progress",True)
		progress.setMinimumDuration(0)
		progress.setCaption(i18n("Creating Photo List"))
		progress.setTotalSteps(len(keys))
		progress.setCancelButtonText(i18n("Cancel"))
		progress.setProgress(0)

		xml = self.lib.getXMLFromKeys(keys, "Thumbs", self.albumName)
		self.__xslXform(xml, self.template, indexHTML)
		
		progress.setCaption(i18n("Exporting Files"))
		for key in keys:
			dict = self.lib.getImage(key)
			progress.setLabelText(i18n("Exporting ")+dict['Caption'])
			(path, filename) = os.path.split(dict['ImagePath'])

			xml = self.lib.getXMLFromKeys([key],"Photo")
			photoHTML = os.path.join(self.exportDir, "p_"+filename+".html")

			# call xsl
			self.__xslXform(xml, self.template, photoHTML)

			# generate images
			thumbPath = os.path.join(thumbDir, filename)
			imagePath = os.path.join(imageDir, filename)
			LUtils.downsizeImage(dict['ImagePath'], thumbPath, QSize(180,180))
			
			if self.size:
				LUtils.downsizeImage(dict['ImagePath'], imagePath, self.size)
			else:
				shutil.copy2(dict['ImagePath'], imagePath)
			progress.setProgress(progress.progress()+1)
			if progress.wasCancelled():
				return
			
		# launch mozilla to this page
		os.system("nslauncher \"file://%s\" &" % (indexHTML))


















