#
# Partial KDE Emulation
#
# This file implements enough of KDE subclasses of Qt to avoid
# the *requirement* for PyKDE in LSongs and LPhoto
#
# THIS IS NOT A GENERAL REPLACEMENT FOR PyKDE
#
# Ideally, the two programs will work with either PyKDE
# or this file - a version running the real KDE will get the
# substantial associated benefits
#
from qt import *

_application = None

def i18n(s):
	global _application
	return unicode(_application.tr(s))

class KAboutData:
	def __init__(self,appName,programName,version,description,license,copyrightStatement,text,homePageAddress,bugsEmailAddress):
		self.appName = appName
		self.programName = programName
		self.version = version
		self.description = description
		self.license = license
		self.copyrightStatement = copyrightStatement
		self.text = text
		self.homePageAddress = homePageAddress
		self.bugsEmailAddress = bugsEmailAddress
		self.authors = []
	
	def addAuthor(self,name,task,emailAddress,webAddress):
		self.authors.append({'name':name,'task':task,'emailAddress':emailAddress,'webAddress':webAddress})
	
	License_GPL = "GPL 2.0"

_cmdLineArgs = None

class KCmdLineArgs:
	def __init__(self):
		pass

	def static_init(args,about):
		global _cmdLineArgs
		_cmdLineArgs = {'args':args,'about':about}
	init = staticmethod(static_init)
	
class KUniqueApplication (QApplication):
	def __init__(self):
		global _application
		QApplication.__init__(self,[])
		_application = self
	
	def static_addCmdLineOptions():
		pass
	addCmdLineOptions = staticmethod(static_addCmdLineOptions)
	
	def quit(self):
		QApplication.quit()

class KListViewItem (QListViewItem):
	def __init__(self,container = None,name = ""):
		QListViewItem.__init__(self,container,name)
		self.m_known = False
		self.m_odd = False
	
	# XXX DSM we do this because of a memory leak in PyQt
	# that leaks profusely when calling key() - so we intercept the
	# sort at a higher level and call it ourselves
	def compare(self, i, col, ascending ):
		key1 = self.key(col,ascending )
		key2 = i.key(col,ascending)
		return cmp(key1,key2)

	def backgroundColor(self):
		if self.isAlternate():
			return self.listView().alternateBackground()
		return self.listView().viewport().colorGroup().base()

	def isAlternate(self):
		lv = self.listView()
		if lv and lv.alternateBackground():
##~ 			if self.itemAbove():
##~ 				return not self.itemAbove().isAlternate()
##~ 			else:
##~ 				return False
			above = self.itemAbove()
			self.m_known = True
			if above:
				self.m_known = above.m_known
			if self.m_known:
				if above:
					self.m_odd = not above.m_odd
				else:
					self.m_odd = False
			else:
				previous = True
				if self.parent():
					item = self.parent()
					if item:
						previous = item.m_odd
					item = self.parent().firstChild()
				else:
					item = lv.firstChild()
				while item:
					item.m_odd = previous
					previous = not previous
					item.m_known = True
					item = item.nextSibling()
			return self.m_odd
		return False
	
	def paintCell(self,p,cg,column,width,alignment):
		pm = None #self.listView().viewport().backgroundPixmap()
		_cg = QColorGroup(cg)
		if pm and not pm.isNull():
			_cg.setBrush(QColorGroup.Base,QBrush(self.backgroundColor(),pm))
			p.setBrushOrigin(-self.listView().contentsX(),-self.listView.contentsY())
		else:
			if self.isAlternate():
				if self.listView().viewport().backgroundMode()==Qt.FixedColor:
					_cg.setColor(QColorGroup.Background,self.listView().alternateBackground())
				else:
					_cg.setColor(QColorGroup.Base,self.listView().alternateBackground())
		QListViewItem.paintCell(self,p,_cg,column,width,alignment)
		

class KListView (QListView):
	def __init__(self,container = None,name = ""):
		QListView.__init__(self,container,name)
		self._dropVisualizer = False
		self._dropHighlighter = False
		self._alternateBackgroundColor = None
		self._fullWidth = False
		self._dragEnabled = False
		self.pressedOnSelected = False

	def setFullWidth(self,bool = True):
		self._fullWidth = bool
		self.header().setStretchEnabled(self.fullWidth(),self.columns()-1)
	
	def fullWidth(self):
		return self._fullWidth

	def setDropVisualizer(self,bool):
		self._dropVisualizer = bool
	
	def dropVisualizer(self):
		return self._dropVisualizer
	
	def setDropHighlighter(self,bool):
		self._dropHighlighter = bool

	def dropHighlighter(self):
		return self._dropHighlighter

	def drawItemHighlighter(self,painter,item):
		r = QRect()
		if item:
			r = self.itemRect(item)
			if painter:
				self.style.drawPrimitive(QStyle.PE_FocusRect,painter,r.self.colorGroup(),QStyle.Style_FocusAtBorder,self.colorGroup().highlight())
		return r
	
	def cleanItemHighlighter(self):
		pass

	def setAlternateBackground(self,color):
		self._alternateBackgroundColor = color
		self.repaint()

	def alternateBackground(self):
		return self._alternateBackgroundColor

	def slotHeaderChanged(self):
		#print "header changed!"
		if self._fullWidth and self.columns:
			w = 0
			for i in xrange(0,self.columns()-1):
				w = w+self.columnWidth(i)
			self.setColumnWidth(self.columns()-1,self.viewport().width()-w-1)

	def contentsMousePress(self,e):
		if self.selectionMode()==QtListView.Extended and ((e.state() & Qt.ShiftButton) and not (e.state() & Qt.ControlButton)):
			block = self.signalsBlocked()
			blockSignals(True)
			self.clearSelection()
			self.blockSignals(block)
		p = self.contentsToViewport(e.pos())
		at = self.itemAt(p)
		rootDecoClicked = False # XXX DSM handle +/- here
		self.validDrag = False
		self.pressedOnSelected = False
		if e.button()==Qt.LeftButton and not rootDecoClicked:
			self.startDragPos = p.pos()
			if at:
				self.validDrag = True
				self.pressedOnSelected = at.isSelected()
		QListView.contentsMousePressEvent(self,e)
	
	def dragging(self,new):
		d = 3
		pos = self.startDragPos
		if new.x()>pos.x()+d: return True
		if new.x()<pos.x()-d: return True
		if new.y()>pos.y()+d: return True
		if new.y()<pos.y()-d: return True
		return False
	
	def setDragEnabled(self,bool):
		self._dragEnabled = bool
	
	def dragEnabled(self):
		return self._dragEnabled
	
	def contentsMouseMoveEvent(self,e):
		if not self.dragEnabled() or self.startDragPos.isNull() or not self.validDrag:
			QListView.contentsMouseMoveEvent(self,e)
		vp = self.contentsToViewport(e.pos())
		item = self.itemAt(vp)
		# XXX DSM cursor change here
		dragOn = self.dragEnabled()
		newPos = e.pos()
		if dragOn and self.validDrag and self.dragging(newPos):
			self.contentsMouseReleaseEvent(None)
			self.startDrag()
			self.startDragPos = QPoint()
			self.validDrag = False

	def contentsMouseReleaseEvent(self,e):
		if e==None:
			QListView.contentsMouseReleaseEvent(None)
		if self.pressedOnSelected and self.itemsRenameable():
			p = self.contentsToViewport(e.pos())
			at = self.itemAt(p)
			if at:
				rootDecoClicked = False # XXX DSM handle +/-
				if not rootDecoClicked:
					col = self.header().mapToLogical(self.header().cellAt(p.x()))
					if self.renameable.contains(col):
						self.rename(at,col)
			self.pressedOnSelected = False
			self.validDrag = False
			self.startDragPos = QPoint()
		QListView.contentsMouseReleaseEvent(self,e)
		
	def contentsDropEvent(self,e):
		self.cleanItemHighlighter()
		if self.acceptDrag(e):
			e.acceptAction()
			(parent,afterme) = self.findDrop(e.pos())
			#if e.source() == self.viewport() and self.itemsMovable():
			#	self.movableDropEvent(parent, afterme)
			#else:
			self.emit(PYSIGNAL("dropped(QDropEvent *,QListViewItem *)"),(e, afterme))
			self.emit(PYSIGNAL("dropped(KListView *,QDropEvent *,QListViewItem)"),(self, e, afterme))
			self.emit(PYSIGNAL("dropped(QDropEvent *,QListViewItem *,QListViewItem *)"),(e, parent, afterme))
			self.emit(PYSIGNAL("dropped(KListView *,QDropEvent *,QListViewItem *,QListViewItem *)"),(self, e, parent, afterme))

	def findDrop(self,pos):
		p  = self.contentsToViewport(pos)
		atpos = self.itemAt(p)
		if not atpos:
			above = self.lastItem()
		else:
			if p.y() - self.itemRect(atpos).topLeft().y() < (atpos.height()/2):
				above = atpos.itemAbove()
			else:
				above = atpos
		if above:
			if above.firstChild() and above.isOpen():
				return (above,None)
			if above.isExpandable():
				if p.x() >= self.depthToPixels(above.depth()+1) or (above.isOpen() and above.hildCount() > 0):
					return (above,None)
			betterAbove = above.parent()
			last = above
			while betterAbove:
				if last.nextSibling() == None:
					if p.x() < depthToPixels ( betterAbove.depth() + 1 ):
						above = betterAbove
					else:
						break
					last = betterAbove
					betterAbove = betterAbove.parent()
				else:
					break
		after = above
		if after: parent = after.parent()
		else: parent = None	
		return (parent,after)

	def dragEnterEvent(self,e):
		#print "drag enter"
		self.contentsDragEnterEvent(e)
		QListView.dragEnterEvent(self,e)

	def dragMoveEvent(self,e):
		#print "drag move"
		self.contentsDragMoveEvent(e)
		QListView.dragMoveEvent(self,e)

	def dropEvent(self,e):
		#print "drag drop"
		self.contentsDropEvent(e)
		QListView.dropEvent(self,e)

class KShortcut:
	def __init__(self,keyCode):
		self.key = keyCode
	
	def static_null():
		return None
	null = staticmethod(static_null)
	
	def keyCode(self):
		return self.key

class KActionCollection:
	def __init__(self,widget = None):
		self.widget = widget
		self.actions = {}
	
	def addAction(self,action):
		self.actions[action.name] = action
	
	def action(self,name):
		return self.actions.get(name,None)

class KAction:
	def __init__(self,text,shortcut,slotFunc,collection,name):
		#print "creating action for",text
		self.text = text
		self.shortcut = shortcut
		self.slotFunc = slotFunc
		self.collection = collection
		self.name = name
		self.collection.addAction(self)
		self.menus = []
	
	def setExclusiveGroup(self,name):
		pass
	
	def plug(self,menu):
		id = menu.insertItem(self.text)
		self.menus.append({'menu':menu,'id':id})
		menu.connectItem(id,self.slotFunc)
		if self.shortcut:
			keyCode = self.shortcut.keyCode()
			if keyCode:
				menu.setAccel(QKeySequence(keyCode),id)
	
	def setEnabled(self,bool):
		for menuItem in self.menus:
			menuItem['menu'].setItemEnabled(menuItem['id'],bool)
	
	def setText(self,text):
		for menuItem in self.menus:
			menuItem['menu'].changeItem(menuItem['id'],text)
	
	def setChecked(self,bool):
		for menuItem in self.menus:
			menuItem['menu'].setItemChecked(menuItem['id'],bool)

class KToggleAction (KAction):
	def __init__(self,text,shortcut,slotFunc,collection,name):
		KAction.__init__(self,text,shortcut,slotFunc,collection,name)

class KRadioAction (KAction):
	def __init__(self,text,shortcut,slotFunc,collection,name):
		KAction.__init__(self,text,shortcut,slotFunc,collection,name)
	
class KStdAction:
	def __init__(self):
		pass
	
	def static_print(slotFunc,collection):
		return KAction(i18n("Print"),KShortcut(Qt.CTRL+Qt.Key_P),slotFunc,collection,"file_print")
	print_ = staticmethod(static_print)

	def static_quit(slotFunc,collection):
		return KAction(i18n("Quit"),KShortcut(Qt.CTRL+Qt.Key_Q),slotFunc,collection,"quit")
	quit = staticmethod(static_quit)

	def static_undo(slotFunc,collection):
		return KAction(i18n("Undo"),KShortcut(Qt.CTRL+Qt.Key_Z),slotFunc,collection,"undo")
	undo = staticmethod(static_undo)

	def static_redo(slotFunc,collection):
		return KAction(i18n("Redo"),KShortcut(Qt.SHIFT+Qt.CTRL+Qt.Key_Z),slotFunc,collection,"redo")
	redo = staticmethod(static_redo)

	def static_cut(slotFunc,collection):
		return KAction(i18n("Cut"),KShortcut(Qt.CTRL+Qt.Key_X),slotFunc,collection,"cut")
	cut = staticmethod(static_cut)

	def static_copy(slotFunc,collection):
		return KAction(i18n("Copy"),KShortcut(Qt.CTRL+Qt.Key_C),slotFunc,collection,"copy")
	copy = staticmethod(static_copy)

	def static_paste(slotFunc,collection):
		return KAction(i18n("Paste"),KShortcut(Qt.CTRL+Qt.Key_V),slotFunc,collection,"paste")
	paste = staticmethod(static_paste)

	def static_preferences(slotFunc,collection):
		return KAction(i18n("Preferences..."), KShortcut.null(), slotFunc, collection, "preferences")
	preferences = staticmethod(static_preferences)

class KFileDialog(QFileDialog):
	pass

class KMessageBox:
	def __init__(self):
		pass
	
	def static_error(widget,text,caption):
		message = QMessageBox.critical(None,caption,text)
	error = staticmethod(static_error)

class KPushButton (QPushButton):
	def __init__(self,container,label):
		QPushButton.__init__(self,container,label)

class KMainWindow(QMainWindow):
	def __init__(self):
		global _cmdLineArgs
		QMainWindow.__init__(self)
		self.setCaption(_cmdLineArgs['about'].appName)
		self.collection = KActionCollection(self)
		
	def actionCollection(self):
		return self.collection
	
	def helpMenu(self,name):
		help = QPopupMenu(self)
		help.insertItem(i18n("&About..."), self.about) #self, SLOT('about()'))
		help.insertItem(i18n("About &Qt"),self.aboutQt) #self, SLOT('aboutQt()'))
		help.insertSeparator()
		help.insertItem(i18n("What's &This"),self, SLOT('whatsThis()'))
		return help

	def about(self):
		pass

	def aboutQt(self):
		QMessageBox.aboutQt( self, i18n("About Qt") )

class KSqueezedTextLabel(QLabel):
	def __init__(self,parent):
		QLabel.__init__(self,"",parent)
		self.fullText = ""
		self.setSizePolicy(QSizePolicy(QSizePolicy.Expanding,QSizePolicy.Fixed))
	
	def setText(self, text):
		self.fullText = text
		self.setSqueezedText()
	
	def text(self):
		return self.fullText
	
	def resizeEvent(self,e):
		self.setSqueezedText()

	def minimumSizeHint(self):
		sh = QLabel.minimumSizeHint(self)
		sh.setWidth(-1)
		return sh

	def setSqueezedText(self):
		fm = QFontMetrics(self.fontMetrics())
		fullText = QString(self.fullText)
		labelWidth = self.size().width()
		textWidth = fm.width(self.fullText)
		if textWidth > labelWidth:
			squeezedText = "..."
			squeezedWidth = fm.width(squeezedText)
			letters = fullText.length() * (labelWidth - squeezedWidth) / textWidth / 2
			if labelWidth < squeezedWidth:
				letters=1
			squeezedText = fullText.left(letters)
			squeezedText.append("...")
			squeezedText.append(fullText.right(letters))
			squeezedWidth = fm.width(squeezedText)
			if squeezedWidth < labelWidth:
				while True:
					letters = letters+1
					squeezedText = fullText.left(letters)
					squeezedText.append("...")
					squeezedText.append(fullText.right(letters))
					squeezedWidth = fm.width(squeezedText)
					if squeezedWidth >= labelWidth:
						break
				letters = letters-1
				squeezedText = fullText.left(letters)
				squeezedText.append("...")
				squeezedText.append(fullText.right(letters))
			elif squeezedWidth > labelWidth:
				while True:
					letters = letters-1
					squeezedText = fullText.left(letters)
					squeezedText.append("...")
					squeezedText.append(fullText.right(letters))
					squeezedWidth = fm.width(squeezedText)
					if not (letters and squeezedWidth > labelWidth):
						break
			if letters < 5:
				QLabel.setText(self,self.fullText)
			else:
				QLabel.setText(self,squeezedText)
			QToolTip.remove(self)
			QToolTip.add(self,fullText)
		else:
			QLabel.setText(self,self.fullText)
			QToolTip.remove(self)
			QToolTip.hide()

class KLineEdit(QLineEdit):
	def __init__(self,text,parent):
		QLineEdit.__init__(self,text,parent)

class KTextEdit(QTextEdit):
	def __init__(self,parent):
		QTextEdit.__init__(self,parent)

class KComboBox(QComboBox):
	def __init__(self,editable,parent):
		QComboBox.__init__(self,editable,parent)

class KRestrictedLine(KLineEdit):
	def __init__(self,parent,name,valid):
		KLineEdit.__init__(self,"",parent)
		self.qsValidChars = valid
		
	def keyPressEvent(self,e):
		if e.key()==Qt.Key_Enter or e.key()==Qt.Key_Return or e.key()==Qt.Key_Delete or e.ascii()<32:
			QLineEdit.keyPressEvent(self,e)
			return
		if self.qsValidChars!=None and not (str(e.text()) in self.qsValidChars):
			# XXX DSM emit signal for invalid char?
			return
		else:
			QLineEdit.keyPressEvent(self,e)
			return

	def setValidChars(self,valid):
		self.sqValidChars = valid
	
	def validChars(self):
		return qsValidChars

class KIconView(QIconView):
	def __init__(self,parent,label):
		QIconView.__init__(self,parent)

class KDialog(QDialog):
	def __init__(self,parent = None,name = "",modal=False,f = 0):
		QDialog.__init__(self,parent,name,modal,f)

class KDialogBase(KDialog):
	def __init__(*args):
		#print "calling KDialogBase with",args,len(args)
		self = args[0]
		if type(args[6]).__name__=='str':
			# tabbed dialog
			KDialog.__init__(self,args[5],args[6],args[7])
			self.setCaption(args[2])
			self.buildTabbedDialog()
		else:
			# untabbed dialog
			KDialog.__init__(self,args[1],args[2],args[3])
			self.setCaption(args[4])
			self.buildNormalDialog()

	def buildDefaultButtons(self,container,containerLayout = None):
		layout = QHBoxLayout()
		layout.setSpacing(10)
		if containerLayout:
			containerLayout.addLayout(layout)
		self.okButton = QPushButton("OK",self,"okbutton")
		self.cancelButton = QPushButton("Cancel",self,"cancelButton")
		layout.addItem(QSpacerItem(1,1,QSizePolicy.Expanding,QSizePolicy.Fixed))
		layout.addWidget(self.okButton)
		layout.addWidget(self.cancelButton)
		QObject.connect(self.okButton,SIGNAL("clicked()"),self.accept)
		QObject.connect(self.cancelButton,SIGNAL("clicked()"),self.reject)
		return layout

	def buildTabbedDialog(self):
		layout = QVBoxLayout(self)
		layout.setSpacing(10)
		layout.setMargin(10)
		self.tabs = QTabWidget(self,"tabs")
		layout.addWidget(self.tabs)
		self.buildDefaultButtons(self,layout)

	def addPage(self,tabName):
		widget = QWidget(self)
		self.tabs.addTab(widget,tabName)
		return widget

	def buildNormalDialog(self):
		self.buttonLayout = self.buildDefaultButtons(self)
	
	def buildLayout(self):
		layout = QVBoxLayout(self)
		layout.setSpacing(10)
		layout.setMargin(10)
		layout.addWidget(self.main)
		layout.addLayout(self.buttonLayout)

	def setMainWidget(self,widget):
		self.main = widget
		self.buildLayout()

	Tabbed = "tabbed"
	Ok = 1
	Cancel = 2
	Apply = 4
