from qt import *
import os, errno, shutil

class LUtils:
	def class_downsizeImage(srcPath, dstPath, size=QSize(240,180)):
		img = QImage(srcPath)
		# align size to image
		if img.width() > img.height() and size.width() < size.height() or img.width() < img.height() and size.width > size.height():
			size.transpose()
		
		if img.width() > size.width() or img.height() > size.height():
			ratio = min(size.width()/float(img.width()), size.height()/float(img.height()))
			w = int(img.width()*ratio)
			h = int(img.height()*ratio)
			t = img.smoothScale(w,h)
			t.save(dstPath, "JPEG", 100)
		else:
			shutil.copy2(srcPath, dstPath)
		return (img.width(), img.height())
	downsizeImage = staticmethod(class_downsizeImage)
	
	def class_getBoundingRects(a, b):
		r1 = QRect(a.left(), a.top(), a.width(), b.top()-a.top())
		r2 = QRect(a.left(), b.top(), b.left()-a.left(), b.height())
		r3 = QRect(b.right()+1, b.top(), a.right()-b.right(), b.height())
		r4 = QRect(a.left(), b.bottom()+1, a.width(), a.bottom()-b.bottom())
		return (r1,r2,r3,r4)
	getBoundingRects = staticmethod(class_getBoundingRects)

	def class_mapRect(ro, ri): #create a new rect that maps ri to the inside of ro
		ratio = min(ro.width()/float(ri.width()), ro.height()/float(ri.height()))
		w = ri.width()*ratio
		h = ri.height()*ratio
		x = ro.x() + (ro.width() - w) / 2
		y = ro.y() + (ro.height() - h) / 2
		return QRect(x,y,w,h)
	mapRect = staticmethod(class_mapRect)
	
	def class_mapRectAndBackground(ro, ri): #create a new rect that maps ri to the inside of ro
		ratio = min(ro.width()/float(ri.width()), ro.height()/float(ri.height()))
		w = ri.width()*ratio
		h = ri.height()*ratio
		x = ro.x() + (ro.width() - w) / 2
		y = ro.y() + (ro.height() - h) / 2
		if w == ro.width():
			b1 = QRect(ro.x(), ro.y(), ro.width(), y)
			b2 = QRect(ro.x(), ro.y()+y+h ,ro.width() , ro.height()-y-h)
		else:
			b1 = QRect(ro.x(), ro.y(), x, ro.height())
			b2 = QRect(ro.x()+x+w, ro.y(), ro.width()-x-w, ro.height())
		return QRect(x,y,w,h), b1, b2
	mapRectAndBackground = staticmethod(class_mapRectAndBackground)

	def class_bound(a, b, c):
		# return value b between a and c
		if (b < a): return a
		elif (b > c): return c
		return b
	bound = staticmethod(class_bound)
	
	def class_centerRect(r, p, b):
		# center's rect(r) on the point(p), bound by the rect(b)
		if r.width() > b.width():
			x = b.width() / 2
		else:
			w = r.width() / 2
			x = LUtils.bound(b.left() + w, p.x(), b.right() - w)
	
		if r.height() > b.height():
			y = b.height() / 2
		else:
			h = r.height() / 2
			y = LUtils.bound(b.top() + h, p.y(), b.bottom() - h)
			
		r.moveCenter(QPoint(x,y))
		return r
	centerRect = staticmethod(class_centerRect)
	
	def class_alignRectToPoint(p1, r, p2, b):
		# position r so that p2 appears at point P1[in r coords]  bound by the rect(b)
		if r.width() > b.width():
			x = (b.width() - r.width()) / 2
		else:
			w = r.width() / 2
			x = LUtils.bound(0, p2.x()-p1.x(), b.right() - r.width())
	
		if r.height() > b.height():
			y = (b.height() - r.height()) / 2
		else:
			h = r.height() / 2
			y = LUtils.bound(0, p2.y()-p1.y(), b.bottom() - r.height())
			
		r.moveTopLeft(QPoint(x,y))
		return r
	alignRectToPoint = staticmethod(class_alignRectToPoint)

	def class_ensureDirectory(path):
		try:
			os.makedirs(path,0777)
		except OSError, err:
			if err.errno != errno.EEXIST or not os.path.isdir(path):
				raise
	ensureDirectory = staticmethod(class_ensureDirectory)
	
	def class_fileExists(path):
		try:
			fd = os.open(path, os.O_RDWR)
		except OSError:
			return False
		os.close(fd)
		return True
	fileExists = staticmethod(class_fileExists)
	
	def class_removeFile(path):
		if LUtils.fileExists(path):
			try:
				os.remove(path)
				return True
			except OSError:
				return False
		return False
	removeFile = staticmethod(class_removeFile)

	def class_humanByteOutput( size):
		if size > 1024 * 1024:
			return str("%0.1f MB)"%(size/float(1024*1024)))
		if size > 1024:
			return str("%0.1f KB"%(size/float(1024)))
	humanByteOutput = staticmethod(class_humanByteOutput)
	
	def class_datePath(time_t):
		dt = QDateTime()
		dt.setTime_t(time_t)
		path = os.path.join(str(dt.date().year()),"%02d"%(dt.date().month()),"%02d"%(dt.date().day()))
		return path
	datePath = staticmethod(class_datePath)
	
	def class_isListChild(parent, child):
		if len(child) < len(parent):
			return False
		return parent[0:len(parent)]==child[0:len(parent)]
	isListChild = staticmethod(class_isListChild)
	
	def class_findUniqueFilename(dir, filename):
		(a,b) = os.path.splitext(filename)
		newName = a + b
		check = str(os.path.join(dir, newName))
		i = 1
		while (LUtils.fileExists(check)):
			newName = a + str(i) + b
			check = os.path.join(dir, newName)
			i = i + 1
		return newName
	findUniqueFilename = staticmethod(class_findUniqueFilename)


global _app

def _setApp(app):
	global _app
	_app = app
	
def _getApp():
	global _app
	return _app

class LWaitCursor:
	setApp = staticmethod(_setApp)
	getApp = staticmethod(_getApp)

	def __init__(self):
		LWaitCursor.getApp().setOverrideCursor(QCursor(Qt.WaitCursor))
	
	def __del__(self):
		LWaitCursor.getApp().restoreOverrideCursor()
		
		
		
		
		
		
